<?php

/**
 * @file
 * Administrative pages for the Data Import Module
 */

/**
 * Implements hook_form().
 */
function data_import_form($form, &$form_state) {
  $form['create_new_import'] = array(
    '#markup' => '<ul class="action-links"><li>' . l(t('Create New Import'), '/admin/config/content/data-import/create') . '</li></ul>',
    '#weight' => 1,
  );

  // Load all importers
  $importers = data_import_load_all_importers();

  if (!empty($importers)) {
    $form['importers'] = array(
      '#weight' => 2,
      '#tree' => TRUE,
      '#theme' => 'data_import_table_form',
    );

    foreach ($importers as $key => $importer) {
      $form['importers'][$key] = array(
        '#type' => 'checkbox',
        '#importer_name' => $importer['name'],
        '#default_value' => $importer['active']
      );
    }

    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save'),
      '#weight' => 3,
    );
  }

  return $form;
}

/**
 * Data import Tanle Theme.
 */
function theme_data_import_table_form($element) {
  $form = $element['form'];

  $header = array(
    'Active',
    'Importer',
    'Actions',
    'Logs');
  $rows = array();

  foreach (element_children($form) as $importer) {
    $rows[] = array(
      array('data' => drupal_render($form[$importer])/* , 'align' => 'center' */),
      array('data' => $form[$importer]['#importer_name'] . ' <b><em>(' . $importer . ')</em></b>'/* , 'align' => 'center' */),
      array('data' => l(t('Run'), 'admin/config/content/data-import/' . $importer . '/run') . ' / '. l(t('Edit'), 'admin/config/content/data-import/config/' . $importer . '/settings') . ' / ' . l(t('Delete'), 'admin/config/content/data-import/config/' . $importer . '/delete')/* , 'align' => 'center' */),
      array('data' => l(t('Log'), 'admin/config/content/data-import/logs', array('query'=>array('importer_id'=> $importer)))/* , 'align' => 'center' */)
    );
  }

  return theme('table', array('header' => $header, 'rows' => $rows));
}

/**
 * Form submit callback function
 */
function data_import_form_submit($form, &$form_state) {
  if (isset($form_state['values']['importers'])) {
    foreach ($form_state['values']['importers'] as $key => $value) {
      $importer = ['importer_id' => $key, 'active' => $value];
      data_importer_save($importer);
    }
  }

  drupal_set_message(t('The configuration have been saved.'));
}
