<?php

/**
 * @file
 * Administrative page callbacks for the Data Import Logging.
 */

 /**
  * Page callback: Displays a listing of database log messages.
  *
  *
  * @param type data_import_log_overview()
  *
  */
function data_import_log_overview() {

  $rows = array();

  $sorting = isset($_GET['sort']) ? $_GET['sort'] : 'desc';

  if(isset($_GET['importer_id'])) {
    $_SESSION['data_import_log_filter']['importer_id'] = $_GET['importer_id'];
  }

  $render['data_import_log_filter'] = drupal_get_form('data_import_log_filter_form');

  $header = array(
    array('data' => t('Process Id'), 'field' => 'log.process_id'),
    array('data' => t('Importer Id'), 'field' => 'log.importer_id'),
    array('data' => t('Date'), 'field' => 'log.timestamp', 'sort' => 'desc'),
    t('Message'),
    array('data' => t('Status'), 'field' => 'log.status'),
  );

  $query = db_select('data_import_log', 'log')->extend('PagerDefault')->extend('TableSort');
  $query
    ->fields('log', array('importer_id', 'process_id', 'status', 'timestamp', 'message'));

  if (isset($_SESSION['data_import_log_filter'])) {
    foreach ($_SESSION['data_import_log_filter'] as $key => $value) {
      if ($value)
        $query->condition($key, $value, '=');
    }
  }

  $result = $query
    ->limit(50)
    ->orderByHeader($header)
    ->orderBy('id', $sorting)
    ->execute();

  foreach ($result as $log) {
    $rows[] = array('data' =>
      array(
        $log->process_id,
        $log->importer_id,
        format_date($log->timestamp, 'short'),
        $log->message,
        $log->status,
      ),
    );
  }

  $render['data_import_log_table'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
    '#empty' => t('No log messages available.'),
  );
  $render['data_import_log_pager'] = array('#theme' => 'pager');

  return $render;
}

/**
 * Implements hook_form().
 */
function data_import_log_filter_form($form, &$form_state) {

  $form['filters'] = array(
    '#type' => 'fieldset',
    '#title' => t('Filter log messages'),
    '#collapsible' => TRUE,
    '#collapsed' => !isset($_SESSION['data_import_log_filter']),
  );

  $form['filters']['process_id'] = array(
    '#title' => t('Process ID :'),
    '#type' => 'textfield',
    '#size' => 20,
    '#default_value' => isset($_SESSION['data_import_log_filter']['process_id']) ? $_SESSION['data_import_log_filter']['process_id'] : ''
  );

  $importers = data_import_load_all_importers();
  $form['filters']['importer_id'] = array(
    '#title' => t('Importer ID :'),
    '#type' => 'select',
    '#options' => array_merge(['all' => t('All')], array_map('importers_map', $importers)),
    '#default_value' => isset($_SESSION['data_import_log_filter']['importer_id']) ? $_SESSION['data_import_log_filter']['importer_id'] : 'all'
  );

  $form['filters']['actions'] = array(
    '#type' => 'actions',
    '#attributes' => array('class' => array('container-inline')),
  );

  $form['filters']['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Filter'),
  );

  $form['filters']['actions']['reset'] = array(
    '#type' => 'submit',
    '#value' => t('Reset')
  );

  return $form;
}

/**
 * Form submission handler for data_import_log_filter_form().
 *
 */
function data_import_log_filter_form_submit($form, &$form_state) {
  $op = $form_state['values']['op'];

  // Reset query params
  if(isset($_GET['importer_id'])) unset($_GET['importer_id']);

  switch ($op) {
    case t('Filter'):
      $_SESSION['data_import_log_filter']['process_id'] = $form_state['values']['process_id'];
      $_SESSION['data_import_log_filter']['importer_id'] = $form_state['values']['importer_id'] == 'all' ? '' : $form_state['values']['importer_id'];
      break;
    case t('Reset'):
      unset($_SESSION['data_import_log_filter']);
      break;
  }
}

/**
* Array importer Map
*/
function importers_map($importer) {
  return $importer['name'];
}
