<?php

/**
* Building the form for the contact setting page.
*/
function data_import_mapping_form($form, &$form_state, $importer_id) {

  // Init all dynamic value
  if (!isset($form_state['#mappings'])) {

    // Importer
    $form_state['#importer'] = data_importer_load($importer_id);

    // Mappings
    $form_state['#mappings'] = $form_state['#origin_mappings'] = load_data_importer_mappings($importer_id);
    $form_state['#key_mappings'] = array_keys($form_state['#mappings']);

    // Content Type
    $form_state['#node'] = [];
    foreach (node_type_get_types() as $key => $type) {
      $form_state['#node'][$key] = $type->name;
    }

    //Taxonomy
    $form_state['#taxonomy_term'] = [];
    foreach (taxonomy_get_vocabularies() as $key => $type) {
      $form_state['#taxonomy_term'][$type->machine_name] = $type->name;
    }

    // Init entity & bundle_name default value
    $entity = ($form_state['#importer'] && $form_state['#importer']['entity']) ? $form_state['#importer']['entity'] : 'node';
    $bundle_name = ($form_state['#importer'] && $form_state['#importer']['bundle_name']) ? $form_state['#importer']['bundle_name'] : array_keys($form_state['#'.$entity])[0];
  }

  // Update entity & Bundle name
  $entity = isset($form_state['values']['entity']) ? $form_state['values']['entity'] : $entity;
  $bundle_name = isset($form_state['values']['bundle_name']) ? $form_state['values']['bundle_name'] : $bundle_name;


  // Update field name option
  $bundle_name = $entity == 'user' ? 'user' : $bundle_name;
  $fields = array_keys(field_info_instances($entity, $bundle_name));

  // Update label & Base fields
  $label = '';
  switch ($entity) {
    case 'node':
      $label = t('Content Type');
      array_unshift($fields, 'title');
      break;

    case 'taxonomy_term':
      $label = t('Vocabulary');
      array_unshift($fields, 'name');
      break;

    case 'user':
      array_unshift($fields, 'pass');
      array_unshift($fields, 'mail');
      array_unshift($fields, 'name');
      break;

    default:
      $label = t('Bundle Name');
      break;
  }

  $form['container'] = array(
    '#type' => 'container',
    '#prefix' => '<div id="mapping-wrapper">',
    '#suffix' => '</div>',
    '#weight' => 0,
  );

  $form['container']['entity'] = array(
    '#type' => 'select',
    '#title' => t('Entity Type'),
    '#options' => array(
      'node' => t('Node'),
      'user' => t('User'),
      'taxonomy_term' => t('Taxonomy')
    ),
    '#default_value' => $entity,
    '#description' => count($form_state['#mappings']) ? t('Remove all rows to edit this.') : t('Please select the Entity type.'),
    '#disabled' => count($form_state['#mappings']) ? TRUE : FALSE,
    '#ajax' => array(
      'callback' => 'ajax_update_mapping_container',
      'wrapper' => 'mapping-wrapper',
    ),
    '#weight' => 0,
  );

// Init ELemet Type
  if ($entity != 'user') {
    $form['container']['bundle_name'] = array(
      '#type' => 'select',
      '#title' => $label,
      '#options' => $form_state['#'.$entity],
      '#default_value' => $bundle_name,
      '#description' => count($form_state['#mappings']) ? t('Remove all rows to edit this.') : t("Please select the @name", array('@name' => $label)),
      '#disabled' => count($form_state['#mappings']) ? TRUE : FALSE,
      '#ajax' => array(
        'callback' => 'ajax_update_mapping_table',
        'wrapper' => 'mappings-table',
      ),
      '#weight' => 0,
    );

  }

  $form['container']['table'] = array(
    '#weight' => 1,
    '#prefix' => '<table id="mappings-table"><tr><th>Source</th><th>Field Name</th><th>Override</th><th>Primary Key</th><th> </th></tr>',
    '#suffix' => '</table>',
    '#description' => 'Source column represents the index of the data you want to import. e.g: enter "0" to represent column 1, "1" to represent column 2, ...'
  );

  $form['container']['help_text'] = array(
    '#markup' => '
    <br/>
    <small>
      <i>* Source column represents the index of the data you want to import. e.g: 
      enter "0" to represent column 1, "1" to represent column 2, ...</i>
      <br/>
      <i>* Check Override to replace content data with import data on next run else import data will be 
      used on content creation only.</i>
      <br/>
      <i>* Primary key is used to identify if content is to be updated or created.</i>
    </small>
    <br/><br/>',
    '#weight' => 2
  );

  $form['container']['table']['rows']['#tree'] = TRUE;

  $primary_key_disabled = FALSE;
  foreach ($form_state['#mappings'] as $mapping) {
    if ($mapping['primary_key'] > 0) {
      $primary_key_disabled = TRUE;
    }
  }

  foreach ($form_state['#mappings'] as $id => $mapping) {

    $mapping += array(
      'importer_id' => $importer_id,
      'source' => '',
      'field_name' => '',
      'override' => 0,
      'primary_key' => 0
    );

    $form['container']['table']['rows'][$id] = array(
      '#tree' => TRUE,
      '#prefix' => '<tr>',
      '#suffix' => '</tr>',
    );

    $form['container']['table']['rows'][$id]['importer_id'] = array(
      '#type' => 'hidden',
      '#value' => $mapping['importer_id'],
    );

    $form['container']['table']['rows'][$id]['source'] = array(
      '#type' => 'textfield',
      '#default_value' => $mapping['source'],
      '#prefix' => '<td>',
      '#suffix' => '</td>',
    );

    $form['container']['table']['rows'][$id]['field_name'] = array(
      '#type' => 'select',
      '#options' => drupal_map_assoc($fields),
      '#default_value' => $mapping['field_name'],
      '#prefix' => '<td>',
      '#suffix' => '</td>',
    );

    $form['container']['table']['rows'][$id]['override'] = array(
      '#type' => 'checkbox',
      '#default_value' => $mapping['override'],
      '#prefix' => '<td>',
      '#suffix' => '</td>',
    );

    $form['container']['table']['rows'][$id]['primary_key'] = array(
      '#type' => 'radio',
      '#name' => 'primary_key',
      '#default_value' => $mapping['primary_key'] ? $id : 0,
      '#return_value' => $id,
      '#disabled' => $primary_key_disabled,
      '#prefix' => '<td>',
      '#suffix' => '</td>',
    );

    $form['container']['table']['rows'][$id]['remove'] = array(
      '#type' => 'submit',
      '#value' => t('Remove'),
      '#button-id' => $id,
      '#name' => 'remove-' . $id,
      '#prefix' => '<td>',
      '#suffix' => '</td>',
      '#submit' => array('remove_mapping_row'),
      '#ajax' => array(
        'callback' => 'ajax_update_mapping_container',
        'wrapper' => 'mapping-wrapper',
      ),
    );
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
    '#weight' => 1,
  );

  $form['add_new'] = array(
    '#type' => 'submit',
    '#value' => t('Add new row'),
    '#submit' => array('add_new_mapping_row'),
    '#ajax' => array(
      'callback' => 'ajax_update_mapping_container',
      'wrapper' => 'mapping-wrapper',
    ),
    '#weight' => 3,
  );

  return $form;
}

/**
* Update container form
*/
function ajax_update_mapping_container($form, $form_state) {
  return $form['container'];
}

/**
* Update Tbale form
*/
function ajax_update_mapping_table($form, $form_state) {
  return $form['container']['table'];
}


/**
* Submit handler for the "add-new-row" button.
* Increments the max counter and causes a rebuild.
*/
function add_new_mapping_row($form, &$form_state) {
  // New item id
  end($form_state['#mappings']);
  $new_id = $form_state['#mappings'] ? key($form_state['#mappings']) + 1 : 1;

  $form_state['#mappings'][$new_id] = array();
  $form_state['rebuild'] = TRUE;
}

/**
* Callback for Remove mapping table row.
*/
function remove_mapping_row($form, &$form_state) {
  $id = $form_state['triggering_element']['#button-id'];
  unset($form_state['#mappings'][$id]);
  $form_state['rebuild'] = TRUE;
}


/**
* Validate form function
*/
function data_import_mapping_form_validate($form, &$form_state) {
  // TO DO
}

/**
* Form submit callback function
*/
function data_import_mapping_form_submit($form, &$form_state) {

  // Updte & Reset Mapping
  if(isset($form_state['values']['rows'])) {
    foreach ($form_state['values']['rows'] as $id => $mapping) {
      //Primary key
      if(isset($_POST['primary_key'])) $mapping['primary_key'] = ($id == $_POST['primary_key']) ? 1 : 0;

      if (in_array($id, $form_state['#key_mappings'])) {
        $mapping['id'] = $id;
        $key = array_search($id, $form_state['#key_mappings']);
        unset($form_state['#key_mappings'][$key]);

        // Update Mapping
        save_data_import_mappings($mapping, 'update');
      } else {

        // Insert New Mapping
        save_data_import_mappings($mapping, 'insert');
      }
    }
  }

  // Remove Mapping
  foreach ($form_state['#key_mappings'] as $key) {
    $mapping['id'] = $key;
    save_data_import_mappings($mapping, 'delete');
  }

  // Update Entity Type
  $form_state['#importer']['entity'] = $form_state['values']['entity'];
  $form_state['#importer']['bundle_name'] = isset($form_state['values']['bundle_name']) ? $form_state['values']['bundle_name'] : '';
  $form_state['#importer']['bundle_name'] = $form_state['values']['entity'] == 'user' ? 'user' : $form_state['#importer']['bundle_name'];
  data_importer_save($form_state['#importer']);

  drupal_set_message(t('The configuration have been saved.'));
}
